#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

update_system() {
    tput cnorm
    local pkgcmd
    if [[ $KERNEL == 'linux-lts' ]]; then
        pkgcmd="pacman -Rs linux --noconfirm ; pacman -S iputils --noconfirm ; pacman -S base-devel git linux-lts --needed --noconfirm"
    else
        pkgcmd="pacman -S iputils --noconfirm ; pacman -S base-devel git --needed --noconfirm"
    fi

    pkgcmd="pacman -Rs archlabs-installer --noconfirm ; $pkgcmd"

    if [[ $KERNEL == 'linux-lts' && $VM ]]; then
        pkgcmd="pacman -Rs virtualbox-guest-modules-arch --noconfirm ; pacman -S virtualbox-guest-dkms linux-lts-headers --noconfirm ; $pkgcmd"
    elif ! [[ $VM ]]; then
        pkgcmd="pacman -Rs virtualbox-guest-utils virtualbox-guest-modules-arch --noconfirm ; $pkgcmd"
    fi

    if [[ $BOOTLOADER != 'grub' ]]; then
        pkgcmd="$pkgcmd ; pacman -Rs grub --noconfirm"
        rm -f $MNT/etc/default/grub
        find $MNT/boot/ -name 'grub*' -exec rm -rf '{}' \; >/dev/null 2>&1
    fi

    chroot_cmd "pacman -Syyu --noconfirm ; $pkgcmd" 2>/dev/null
    return 0
}

install_packages() {
    tput civis
    # packages needed for the selected window manager
    for wm in $INSTALL_WMS; do
        WM_LOGIN_CHOICES="${WM_LOGIN_CHOICES}$wm - "
        case $wm in
            openbox)  WM_PACKAGES="$WM_PACKAGES $wm obconf archlabs-obkey archlabs-kickshaw tint2 conky" ;;
            bspwm)    WM_PACKAGES="$WM_PACKAGES $wm sxhkd" ;;
            i3-gaps)  WM_PACKAGES="$WM_PACKAGES $wm i3status perl-anyevent-i3" ;;
            gnome)    WM_PACKAGES="$WM_PACKAGES $wm gnome-extra" ;;
            cinnamon) WM_PACKAGES="$WM_PACKAGES $wm" ;;
            xfce4)    WM_PACKAGES="$WM_PACKAGES $wm xfce4-goodies xfce4-pulseaudio-plugin" ;;
        esac
    done

    if [[ $INSTALL_WMS =~ (openbox|bspwm|i3-gaps) ]]; then
        WM_PACKAGES="$WM_PACKAGES termite thunar archlabs-polybar archlabs-screenlock archlabs-skippy-xd archlabs-oblogout lxappearance"
    elif [[ $INSTALL_WMS =~ (xfce4) ]]; then
        WM_PACKAGES="$WM_PACKAGES archlabs-oblogout archlabs-screenlock"
    fi

    local pkgs="$WM_PACKAGES"

    # add lightdm packages if chosen
    if [[ $LOGIN_TYPE == 'lightdm' ]]; then
        pkgs="$pkgs lightdm lightdm-gtk-greeter lightdm-gtk-greeter-settings accountsservice"
    fi

    # add extra packages
    [[ $EXTRA_PACKAGES != "" ]] && pkgs="$pkgs $EXTRA_PACKAGES"

    # for gnome and cinnamon we dont need the xfce provided stuff
    if [[ $INSTALL_WMS == 'gnome' || $INSTALL_WMS == 'cinnamon' ]]; then
        REMOVE_PKGS="$(pacman -Qssq 'xfce4*' 2>/dev/null)"
    fi

    [[ $INSTALL_WMS =~ dwm ]] && setup_dwm

    local pkgcmd="pacman -S $pkgs --needed --noconfirm"

    # are we removing some packages
    [[ $REMOVE_PKGS != "" ]] && pkgcmd="pacman -Rs $REMOVE_PKGS --noconfirm ; $pkgcmd"

    chroot_cmd "$pkgcmd" 2>/dev/null
    return 0
}

setup_dwm() {
    echo -e "\nSetting up dwm..\n\n"
    mkdir -pv $MNT/home/$LIVE/suckless
    for prog in dwm st dmenu; do
        echo -e "\nInstalling $prog..\n"
        git clone https://bitbucket.org/natemaia/$prog $MNT/home/$LIVE/suckless/$prog
        chroot_cmd "cd /home/$LIVE/suckless/$prog && sudo make clean install && make clean"
        echo -e "\n$prog has been installed..
        \nSee /home/$NEWUSER/suckless/$prog to customize and rebuild\n"
    done
}

setup_lightdm() {
    # due to the user's information not being entered yet at this point, if they chose
    # autologin then there is one value in the lightdm config that must be changed later
    #
    #        autologin-user=$USER

    echo -e "\nSetting up lightdm..\n"
    chroot_cmd 'systemctl enable lightdm.service && systemctl set-default graphical.target'

    local cfg="$MNT/etc/lightdm/lightdm-gtk-greeter.conf"

    sed -i '/#background=/ c background=/usr/share/backgrounds/archlabs/archlabs.jpg' $cfg
    sed -i '/#theme-name=/ c theme-name=ArchLabs-dARK' $cfg
    sed -i '/#icon-theme-name=/ c icon-theme-name=ArchLabs-Light' $cfg
    sed -i '/#position=/ c position=34%,end 66%,end' $cfg
    sed -i '/#font-name=/ c font-name=DejaVu Sans Mono 11' $cfg
    sed -i '/\[greeter]/ a default-user-image=/usr/share/icons/ArchLabs-Dark/64x64/places/distributor-logo-archlabs.png' $cfg
    sed -i '/\[greeter]/ a active-monitor=0' $cfg

    rm -rf $MNT/etc/systemd/system/getty@tty1.service.d

    if [[ $AUTOLOGIN == true ]]; then
        echo -e "\nSetting up autologin..\n"
        chroot_cmd 'groupadd -r nopasswdlogin'
        sed -i '/#%PAM-1.0/ a auth        sufficient  pam_succeed_if.so user ingroup nopasswdlogin' $MNT/etc/pam.d/lightdm
        sed -i "/#autologin-session=/ c autologin-session=${LOGIN_WM}" $MNT/etc/lightdm/lightdm.conf
        sed -i "/#autologin-user=/ c autologin-user=${NEWUSER}" $MNT/etc/lightdm/lightdm.conf
    fi
}
